import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot';

// NY Times US rolling averages
// https://github.com/nytimes/covid-19-data/blob/master/rolling-averages/us-states.csv
// https://github.com/nytimes/covid-19-data/blob/master/rolling-averages/us.csv
const csvUrl = [
  'https://raw.githubusercontent.com/nytimes/covid-19-data/master/rolling-averages/us.csv',
].join('');

// date,geoid,cases,cases_avg,cases_avg_per_100k,deaths,deaths_avg,deaths_avg_per_100k
const parseRow = (d) => {
  d.date = d.date;
  d.geoid = d.geoid;
  d.state = d.state;
  d.cases = +d.cases;
  d.cases_avg = +d.cases_avg;
  d.cases_avg_per_100k = +d.cases_avg_per_100k;
  d.deaths = +d.deaths;
  d.deaths_avg = +d.deaths_avg;
  d.deaths_avg_per_100k = +d.deaths_avg_per_100k;
  return d;
};

const width = window.innerWidth;
const height = window.innerHeight;
const yAxisLabel = 'deaths_avg';
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

const main = async () => {
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(await csv(csvUrl, parseRow))
    .xValue((d) => d.date)
    .yValue((d) => d.deaths_avg)
    .yAxisLabel(yAxisLabel)
    .margin({
      top: 100,
      right: 100,
      bottom: 100,
      left: 100,
    })
    .radius(2);

  svg.call(plot);

  const columns = ['cases_avg', 'deaths_avg'];
  let i = 0;
  setInterval(() => {
    i++;
    plot
      .yValue((d) => d[columns[i % columns.length]])
      .yAxisLabel(columns[i % columns.length]);
    svg.call(plot);
  }, 2400);
};
main();

