import {
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  timeFormat,
  transition,
} from 'd3';

export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let yAxisLabel;
  let margin;
  let radius;

  const my = (selection) => {
    data.forEach((d) => (d.date = new Date(d.date)));

    const x = scaleTime()
      .domain(extent(data, (d) => d.date))
      .range([100, width - 50])
      .nice();

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top])
      .nice();

    const axisBottomLabelMonths = axisBottom(x)
      .ticks(16)
      .tickFormat(timeFormat('%b'));

    const axisBottomLabelYear = axisBottom(x)
      .ticks(d3.timeYear.every(1))
      .tickPadding([30])
      .tickFormat(timeFormat('%Y'));

    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
    }));

    const growRadiusDuration = transition().duration(1);

    const t = transition().duration(1);

    const positionCircles = (circles) => {
      circles.attr('cx', (d) => d.x).attr('cy', (d) => d.y);
    };

    const initializeRadius = (circles) => {
      circles.attr('r', 0);
    };

    const growRadius = (enter) => {
      enter
        .transition(growRadiusDuration)
        .attr('r', radius);
    };

    const addAxisLabelY = (add) => {
      add.yAxisLabel;
    };

    const circles = selection
      .selectAll('circle')
      .data(marks)
      .join(
        (enter) =>
          enter
            .append('circle')
            .call(positionCircles)
            .call(initializeRadius)
            .call(growRadius)
            .style('fill', 'darkgray'),
        (update) =>
          update.call((update) =>
            update
              .transition(t)
              .delay((d, i) => i * 2)
              .call(positionCircles)
              .style('fill', 'darkgray')
          ),
        (exit) => exit.remove()
      );

    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .transition(t)
      .call(axisLeft(y));

    selection
      .selectAll('.y-axis-label')
      .data([null])
      .join(
        (enter) =>
          enter
            .append('text')
            .attr('class', 'y-axis-label')
            .text(yAxisLabel)
            .attr('x', margin.left - 50)
            .attr('y', -10)
            .attr('font-weight', 'bold')
            .attr('font-size', '18')
            .attr('fill', 'midnightblue')
            .call((enter) =>
              enter.transition(t).attr('y', margin.top - 21)
            ),
        (update) =>
          update.call((update) =>
            update
              .attr('y', -10)
              .transition(t)
              .text(yAxisLabel)
              .attr('y', margin.top - 21)
              .attr('font-weight', 'bold')
              .attr('font-size', '18')
              .attr('fill', 'midnightblue')
          ),
        (exit) => exit.remove()
      );

    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .transition(t)
      .call(axisBottomLabelMonths);

    selection
      .append('g')
      .attr('class', 'axisBottomYear')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .call(axisBottomLabelYear);

    selection
      .append('text')
      .attr(
        'transform',
        'translate(' +
          width / 2 +
          ' ,' +
          (margin.left - 20) +
          ')'
      )
      .attr('font-weight', 'bold')
      .attr('font-size', '18')
      .attr('fill', 'midnightblue')
      .style('text-anchor', 'middle')
      .text('UNITED STATES');

    selection
      .append('text')
      .attr(
        'transform',
        'translate(' +
          width * 0.9 +
          ' ,' +
          (margin.right + 275) +
          ')'
      )
      .attr('font-weight', 'bold')
      .attr('font-size', '18')
      .attr('fill', 'red')
      .style('text-anchor', 'middle')
      .text('Vaccination >40%');

    selection
      .append('text')
      .attr(
        'transform',
        'translate(' +
          width / 2 +
          ' ,' +
          (height - margin.bottom + 80) +
          ')'
      )
      .attr('font-size', '18')
      .attr('fill', 'gray')
      .style('text-anchor', 'middle')
      .text(
        'source NYT: https://github.com/nytimes/covid-19-data'
      );

    selection
      .append('text')
      .attr(
        'transform',
        'translate(' + 90 + ' ,' + (margin.left - 45) + ')'
      )
      .attr('font-weight', 'bold')
      .attr('font-size', '20')
      .attr('fill', 'midnightblue')
      .style('text-anchor', 'middle')
      .text('COVID-19');
  };

  // accessor properties - getters and setters:
  // functions that execute on getting and setting a value, but look like regular properties to an external code.

  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.yAxisLabel = function (_) {
    return arguments.length
      ? ((yAxisLabel = _), my)
      : yAxisLabel;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};

// TODO make it idempotent
